----------------------------------------------------------------------------------
-- Company: AGH
-- Engineer: ukasz Jele
-- 
-- Create Date: 09.02.2017 08:55:57

-- Module Name: Deserializer
-- Project Name: RobotEYE
-- Target Devices: Nexys Video
-- Description: 
--      This module Deserializes serial data.
--
-- Additional Comments:
--      None
----------------------------------------------------------------------------------



library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity deserializer is
    Generic (
        Port_width      : integer := 10         -- (2-8,10,14)
    );
    Port ( 
        clk_ser         : in    STD_LOGIC := '0';
        clk_div         : in    STD_LOGIC := '0';
        serial_in       : in    STD_LOGIC := '0';
        parallel_out    : out   STD_LOGIC_VECTOR( Port_width - 1 downto 0 );
        
        bit_slip_in     : in    STD_LOGIC := '0';
        clk_en_in       : in    STD_LOGIC := '0';
        reset_in        : in    STD_LOGIC := '0'
    );
end deserializer;

architecture Behavioral of deserializer is

    signal data_rec     : STD_LOGIC_VECTOR( 13 downto 0 ) := (others => '0');
    
    signal shift1       : STD_LOGIC;
    signal shift2       : STD_LOGIC;
           
    signal clk_ser_n    : STD_LOGIC;
    
begin


    clk_ser_n   <= not clk_ser;


reverse:
    for I in 0 to Port_width-1 generate
    begin
        parallel_out(I) <= data_rec(Port_width-I);
    end generate;
    

ISERDESE2_master: ISERDESE2
   generic map (
      DATA_RATE => "DDR",                       -- DDR, SDR
      DATA_WIDTH => Port_width,                 -- Parallel data width (2-8,10,14)
      DYN_CLKDIV_INV_EN => "FALSE",             -- Enable DYNCLKDIVINVSEL inversion (FALSE, TRUE)
      DYN_CLK_INV_EN => "FALSE",                -- Enable DYNCLKINVSEL inversion (FALSE, TRUE)
      -- INIT_Q1 - INIT_Q4: Initial value on the Q outputs (0/1)
      INIT_Q1 => '0',
      INIT_Q2 => '0',
      INIT_Q3 => '0',
      INIT_Q4 => '0',
      INTERFACE_TYPE => "NETWORKING",           -- MEMORY, MEMORY_DDR3, MEMORY_QDR, NETWORKING, OVERSAMPLE
      IOBDELAY => "NONE",                       -- NONE, BOTH, IBUF, IFD
      NUM_CE => 1,                              -- Number of clock enables (1,2)
      OFB_USED => "FALSE",                      -- Select OFB path (FALSE, TRUE)
      SERDES_MODE => "MASTER",                  -- MASTER, SLAVE
      -- SRVAL_Q1 - SRVAL_Q4: Q output values when SR is used (0/1)
      SRVAL_Q1 => '0',
      SRVAL_Q2 => '0',
      SRVAL_Q3 => '0',
      SRVAL_Q4 => '0' 
   )
   port map (
      O => open,                                -- 1-bit output: Combinatorial output
      -- Q1 - Q8: 1-bit (each) output: Registered data outputs
      Q1 => data_rec(0),
      Q2 => data_rec(1),
      Q3 => data_rec(2),
      Q4 => data_rec(3),
      Q5 => data_rec(4),
      Q6 => data_rec(5),
      Q7 => data_rec(6),
      Q8 => data_rec(7),
      -- SHIFTOUT1, SHIFTOUT2: 1-bit (each) output: Data width expansion output ports
      SHIFTOUT1 => shift1,
      SHIFTOUT2 => shift2,
      BITSLIP => bit_slip_in,                   -- 1-bit input: The BITSLIP pin performs a Bitslip operation synchronous to
                                                -- CLKDIV when asserted (active High). Subsequently, the data seen on the
                                                -- Q1 to Q8 output ports will shift, as in a barrel-shifter operation, one
                                                -- position every time Bitslip is invoked (DDR operation is different from
                                                -- SDR).

      -- CE1, CE2: 1-bit (each) input: Data register clock enable inputs
      CE1 => clk_en_in,
      CE2 => '1',
      CLKDIVP => '0',                           -- 1-bit input: TBD
      -- Clocks: 1-bit (each) input: ISERDESE2 clock input ports
      CLK => clk_ser,                           -- 1-bit input: High-speed clock
      CLKB => clk_ser_n,                        -- 1-bit input: High-speed secondary clock
      CLKDIV => clk_div,                        -- 1-bit input: Divided clock
      OCLK => '0',                              -- 1-bit input: High speed output clock used when INTERFACE_TYPE="MEMORY" 
      -- Input Data: 1-bit (each) input: ISERDESE2 data input ports
      DYNCLKDIVSEL => '0',
      DYNCLKSEL    => '0',
      D => serial_in,                           -- 1-bit input: Data input
      DDLY => '0',                              -- 1-bit input: Serial data from IDELAYE2
      OFB => '0',                               -- 1-bit input: Data feedback from OSERDESE2
      OCLKB => '0',                             -- 1-bit input: High speed negative edge output clock
      RST => reset_in,                          -- 1-bit input: Active high asynchronous reset
      -- SHIFTIN1, SHIFTIN2: 1-bit (each) input: Data width expansion input ports
      SHIFTIN1 => '0',
      SHIFTIN2 => '0' 
   );

ISERDESE2_slave: ISERDESE2
   generic map (
      DATA_RATE => "DDR",                       -- DDR, SDR
      DATA_WIDTH => Port_width,                 -- Parallel data width (2-8,10,14)
      DYN_CLKDIV_INV_EN => "FALSE",             -- Enable DYNCLKDIVINVSEL inversion (FALSE, TRUE)
      DYN_CLK_INV_EN => "FALSE",                -- Enable DYNCLKINVSEL inversion (FALSE, TRUE)
      -- INIT_Q1 - INIT_Q4: Initial value on the Q outputs (0/1)
      INIT_Q1 => '0',
      INIT_Q2 => '0',
      INIT_Q3 => '0',
      INIT_Q4 => '0',
      INTERFACE_TYPE => "NETWORKING",           -- MEMORY, MEMORY_DDR3, MEMORY_QDR, NETWORKING, OVERSAMPLE
      IOBDELAY => "NONE",                       -- NONE, BOTH, IBUF, IFD
      NUM_CE => 1,                              -- Number of clock enables (1,2)
      OFB_USED => "FALSE",                      -- Select OFB path (FALSE, TRUE)
      SERDES_MODE => "SLAVE",                   -- MASTER, SLAVE
      -- SRVAL_Q1 - SRVAL_Q4: Q output values when SR is used (0/1)
      SRVAL_Q1 => '0',
      SRVAL_Q2 => '0',
      SRVAL_Q3 => '0',
      SRVAL_Q4 => '0' 
   )
   port map (
      O => open,                                -- 1-bit output: Combinatorial output
      -- Q1 - Q8: 1-bit (each) output: Registered data outputs
      Q1 => open,
      Q2 => open,
      Q3 => data_rec(8),
      Q4 => data_rec(9),
      Q5 => data_rec(10),
      Q6 => data_rec(11),
      Q7 => data_rec(12),
      Q8 => data_rec(13),
      -- SHIFTOUT1, SHIFTOUT2: 1-bit (each) output: Data width expansion output ports
      SHIFTOUT1 => open,
      SHIFTOUT2 => open,
      BITSLIP => bit_slip_in,                   -- 1-bit input: The BITSLIP pin performs a Bitslip operation synchronous to
                                                -- CLKDIV when asserted (active High). Subsequently, the data seen on the
                                                -- Q1 to Q8 output ports will shift, as in a barrel-shifter operation, one
                                                -- position every time Bitslip is invoked (DDR operation is different from
                                                -- SDR).

      -- CE1, CE2: 1-bit (each) input: Data register clock enable inputs
      CE1 => clk_en_in,
      CE2 => '1',
      CLKDIVP => '0',                           -- 1-bit input: TBD
      -- Clocks: 1-bit (each) input: ISERDESE2 clock input ports
      CLK => clk_ser,                           -- 1-bit input: High-speed clock
      CLKB => clk_ser_n,                        -- 1-bit input: High-speed secondary clock
      CLKDIV => clk_div,                        -- 1-bit input: Divided clock
      OCLK => '0',                              -- 1-bit input: High speed output clock used when INTERFACE_TYPE="MEMORY" 
      -- Input Data: 1-bit (each) input: ISERDESE2 data input ports
      DYNCLKDIVSEL => '0',
      DYNCLKSEL    => '0',
      D => '0',                           -- 1-bit input: Data input
      DDLY => '0',                        -- 1-bit input: Serial data from IDELAYE2
      OFB => '0',                               -- 1-bit input: Data feedback from OSERDESE2
      OCLKB => '0',                             -- 1-bit input: High speed negative edge output clock
      RST => reset_in,                          -- 1-bit input: Active high asynchronous reset
      -- SHIFTIN1, SHIFTIN2: 1-bit (each) input: Data width expansion input ports
      SHIFTIN1 => shift1,
      SHIFTIN2 => shift2 
   );

end Behavioral;
