----------------------------------------------------------------------------------
-- Company: AGH
-- Engineer: ukasz Jele
--
-- Create Date: 09.02.2017 08:47:34
-- 
-- Module Name: DVI_to_VGA
-- Project Name: RobotEYE
-- Target Devices: Nexys Video
-- Description: 
--      This module converts DVI signal to VGA
--
-- Dependencies: 
--      None
--
-- Additional Comments:
--      None
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity DVI_to_VGA is
    Port ( 
        reset_in        : in    STD_LOGIC;
        
        DVI_clk_in      : in    STD_LOGIC;
        DVI_ch_in       : in    STD_LOGIC_VECTOR(2 downto 0);
        
        VGA_clk_out     : out   STD_LOGIC := '0';
        VGA_red         : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        VGA_green       : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        VGA_blue        : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
        VGA_hsync       : out   STD_LOGIC := '0';
        VGA_vsync       : out   STD_LOGIC := '0';
        VGA_blank       : out   STD_LOGIC := '0'
    );
end DVI_to_VGA;

architecture Behavioral of DVI_to_VGA is


--##--##--##--##--## Clocking ##--##--##--##--##--

    component dvi_deser_clk
        port (
            DVI_clk_div         : out    std_logic;
            DVI_clk_diff        : out    std_logic;
            -- Status and control signals
            reset               : in     std_logic;
            locked              : out    std_logic;
            DVI_clk_in          : in     std_logic
        );
    end component;
    
    
--##--##--##--##--## Bit Slipper ##--##--##--##--##--

    component bit_slipper is
        Port (                                        
            signal clk_in           : in    STD_LOGIC;
            signal err_in           : in    STD_LOGIC;
            signal slip_bit_out     : out   STD_LOGIC;
            signal sync_done_out    : out   STD_LOGIC;
            signal reset_in         : in    STD_LOGIC 
        );                                            
    end component bit_slipper;
    
    
--##--##--##--##--## Deserializer ##--##--##--##--##--
 
    component deserializer is
        Generic (
            Port_width      : integer := 10         -- (2-8,10,14)
        );
        Port ( 
            clk_ser         : in    STD_LOGIC := '0';
            clk_div         : in    STD_LOGIC := '0';
            serial_in       : in    STD_LOGIC := '0';
            parallel_out    : out   STD_LOGIC_VECTOR( Port_width - 1 downto 0 );
            
            bit_slip_in     : in    STD_LOGIC := '0';
            clk_en_in       : in    STD_LOGIC := '0';
            reset_in        : in    STD_LOGIC := '0'
        );
    end component deserializer;


    --##--##--##-- Convert --##--##--##--
    component dvi_decoder is
        Port ( 
            DVI_clk         : in    STD_LOGIC;
            DVI_ch_b        : in    STD_LOGIC_VECTOR(9 downto 0);
            DVI_ch_g        : in    STD_LOGIC_VECTOR(9 downto 0);
            DVI_ch_r        : in    STD_LOGIC_VECTOR(9 downto 0);
            err_b           : out   STD_LOGIC;
            err_g           : out   STD_LOGIC;
            err_r           : out   STD_LOGIC;
            
            VGA_red         : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_green       : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_blue        : out   STD_LOGIC_VECTOR(7 downto 0) := (others => '0');
            VGA_hsync       : out   STD_LOGIC := '0';
            VGA_vsync       : out   STD_LOGIC := '0';
            VGA_blank       : out   STD_LOGIC := '0';
            HCNT            : out   STD_LOGIC_VECTOR(11 downto 0);
            VCNT            : out   STD_LOGIC_VECTOR(11 downto 0);
            
            reset_in        : in    STD_LOGIC
        );
    end component dvi_decoder;
    
    
    
--##--##--##--##--## GLOBAL Signal ##--##--##--##--##--

--**-- Clocking signals --**-- 
    signal w_clk_div        : STD_LOGIC;
    signal w_clk_diff       : STD_LOGIC;
    
--**-- Data signals --**-- 
    signal w_clk_locked     : STD_LOGIC;
    signal w_clk_locked_n   : STD_LOGIC;
    
--**-- DeSerializer --**--
    signal data_recv_b      : STD_LOGIC_VECTOR(9 downto 0);
    signal data_recv_g      : STD_LOGIC_VECTOR(9 downto 0);
    signal data_recv_r      : STD_LOGIC_VECTOR(9 downto 0);
            
    signal slip_bit_b       : STD_LOGIC;
    signal slip_bit_g       : STD_LOGIC;
    signal slip_bit_r       : STD_LOGIC;
    
    signal sync_done_b      : STD_LOGIC;
    signal sync_done_g      : STD_LOGIC;
    signal sync_done_r      : STD_LOGIC;
    signal sync_done        : STD_LOGIC;
    
    signal errors           : STD_LOGIC_VECTOR(2 downto 0);
    
begin

--##--##--##--##--## GLOBAL conn ##--##--##--##--##--
    
    w_clk_locked_n  <= not w_clk_locked;
    
    VGA_clk_out     <= w_clk_div;
    
              
    --##--##--##-- Delay --##--##--##--
      
    
tmds_ch0_b: 
    deserializer Generic map(
        Port_width          => 10         -- (2-8,10,14)
    )Port map( 
        clk_ser             => w_clk_diff,
        clk_div             => w_clk_div,
        serial_in           => DVI_ch_in(0),
        parallel_out        => data_recv_b,
        
        bit_slip_in         => slip_bit_b,
        clk_en_in           => w_clk_locked,
        reset_in            => w_clk_locked_n
    );

tmds_ch1_g: 
    deserializer Generic map(
        Port_width          => 10         -- (2-8,10,14)
    )Port map( 
        clk_ser             => w_clk_diff,
        clk_div             => w_clk_div,
        serial_in           => DVI_ch_in(1),
        parallel_out        => data_recv_g,
        
        bit_slip_in         => slip_bit_g,
        clk_en_in           => w_clk_locked,
        reset_in            => w_clk_locked_n
    );

tmds_ch2_r: 
    deserializer Generic map(
        Port_width          => 10         -- (2-8,10,14)
    )Port map( 
        clk_ser             => w_clk_diff,
        clk_div             => w_clk_div,
        serial_in           => DVI_ch_in(2),
        parallel_out        => data_recv_r,
        
        bit_slip_in         => slip_bit_r,
        clk_en_in           => w_clk_locked,
        reset_in            => w_clk_locked_n
    );
    
                                   
    
bit_slip_b: bit_slipper Port map(         
        clk_in              => w_clk_div,
        err_in              => errors(0),
        slip_bit_out        => slip_bit_b,
        sync_done_out       => sync_done_b,
        reset_in            => w_clk_locked_n
    );
    
bit_slip_g: bit_slipper Port map(         
        clk_in              => w_clk_div,
        err_in              => errors(1),
        slip_bit_out        => slip_bit_g,
        sync_done_out           => sync_done_g,
        reset_in            => w_clk_locked_n
    );
        
bit_slip_r: bit_slipper Port map(         
        clk_in              => w_clk_div,
        err_in              => errors(2),
        slip_bit_out        => slip_bit_r,
        sync_done_out       => sync_done_r,
        reset_in            => w_clk_locked_n
    );

    sync_done <= not (sync_done_b and sync_done_g and sync_done_r);


converter: dvi_decoder Port map ( 
        DVI_clk             => w_clk_div,
        DVI_ch_b            => data_recv_b,
        DVI_ch_g            => data_recv_g,
        DVI_ch_r            => data_recv_r,
        err_b               => errors(0),
        err_g               => errors(1),
        err_r               => errors(2),
        
        VGA_red             => VGA_red,
        VGA_green           => VGA_green,
        VGA_blue            => VGA_blue,
        VGA_hsync           => VGA_hsync,
        VGA_vsync           => VGA_vsync,
        VGA_blank           => VGA_blank,
        HCNT                => open,
        VCNT                => open,
        
        reset_in            => sync_done
    );
    
    
--##--##--##--##--## Clocking ##--##--##--##--##--   
    
u_deser_clk: 
    dvi_deser_clk port map ( 
        -- Clock out ports  
        DVI_clk_div         => w_clk_div,
        DVI_clk_diff        => w_clk_diff,       
        reset               => reset_in,
        locked              => w_clk_locked,
        DVI_clk_in          => DVI_clk_in
    );


end Behavioral;
