----------------------------------------------------------------------------------
-- Company: AGH
-- Engineer: ukasz Jele
-- 
-- Create Date: 28.01.2017 07:25:28

-- Module Name: serializer
-- Project Name: RobotEYE
-- Target Devices: Nexys Video
-- Description: 
--      This module serializes parallel data.
--
-- Additional Comments:
--      None
----------------------------------------------------------------------------------


--component serializer is
--    Generic (
--        Port_width      : integer := 10         -- (2-8,10,14)
--    );
--    Port ( 
--        clk_ser         : in    STD_LOGIC;
--        clk_div         : in    STD_LOGIC;
--        parallel_in     : in    STD_LOGIC_VECTOR( Port_width - 1 downto 0 );
--        serial_out      : out   STD_LOGIC;
--        reset           : in    STD_LOGIC
--    );
--end component serializer;

--Ux: serializer generic map(
--    Port_width      => 10
--) port map (
--    clk_ser         => ,
--    clk_div         => ,
--    parallel_in     => ,
--    serial_out      => ,
--    reset_in        => 
--);

----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
library UNISIM;
use UNISIM.VComponents.all;


entity serializer is
    Generic (
        Port_width      : integer := 10         -- (2-8,10,14)
    );
    Port ( 
        clk_ser         : in    STD_LOGIC;
        clk_div         : in    STD_LOGIC;
        parallel_in     : in    STD_LOGIC_VECTOR( Port_width - 1 downto 0 );
        serial_out      : out   STD_LOGIC;
        reset_in        : in    STD_LOGIC
    );
end serializer;

architecture Behavioral of serializer is

    signal data_to_send : STD_LOGIC_VECTOR( 13 downto 0 ) := (others => '0');
    
    signal shift1,
           shift2       : STD_LOGIC;

begin

    process(clk_div)
    begin
        if rising_edge(clk_div) then
            data_to_send( Port_width - 1 downto 0 ) <= parallel_in;
        end if;
    end process;

OSERDESE2_master: OSERDESE2
generic map (
    DATA_RATE_OQ    => "DDR",       -- DDR, SDR
    DATA_RATE_TQ    => "SDR",       -- DDR, BUF, SDR
    DATA_WIDTH      => Port_width,  -- Parallel data width (2-8,10,14)
    INIT_OQ         => '0',         -- Initial value of OQ output (1'b0,1'b1)
    INIT_TQ         => '0',         -- Initial value of TQ output (1'b0,1'b1)
    SERDES_MODE     => "MASTER",    -- MASTER, SLAVE
    SRVAL_OQ        => '0',         -- OQ output value when SR is used (1'b0,1'b1)
    SRVAL_TQ        => '0',         -- TQ output value when SR is used (1'b0,1'b1)
    TBYTE_CTL       => "FALSE",     -- Enable tristate byte operation (FALSE, TRUE)
    TBYTE_SRC       => "FALSE",     -- Tristate byte source (FALSE, TRUE)
    TRISTATE_WIDTH  => 1            -- 3-state converter width (1,4)
) port map (
    OFB             => open,        -- 1-bit output: Feedback path for data
    OQ              => serial_out,  -- 1-bit output: Data path output
    -- SHIFTOUT1 / SHIFTOUT2: 1-bit (each) output: Data output expansion (1-bit each)
    SHIFTOUT1       => open,
    SHIFTOUT2       => open,
    TBYTEOUT        => open,        -- 1-bit output: Byte group tristate
    TFB             => open,        -- 1-bit output: 3-state control
    TQ              => open,        -- 1-bit output: 3-state control
    CLK             => clk_ser,     -- 1-bit input: High speed clock
    CLKDIV          => clk_div,     -- 1-bit input: Divided clock
    -- D1 - D8: 1-bit (each) input: Parallel data inputs (1-bit each)
    D1              => data_to_send(0),
    D2              => data_to_send(1),
    D3              => data_to_send(2),
    D4              => data_to_send(3),
    D5              => data_to_send(4),
    D6              => data_to_send(5),
    D7              => data_to_send(6),
    D8              => data_to_send(7),
    OCE             => '1',         -- 1-bit input: Output data clock enable
    RST             => reset_in,       -- 1-bit input: Reset
    -- SHIFTIN1 / SHIFTIN2: 1-bit (each) input: Data input expansion (1-bit each)
    SHIFTIN1        => shift1,
    SHIFTIN2        => shift2,
    -- T1 - T4: 1-bit (each) input: Parallel 3-state inputs
    T1              => '0',
    T2              => '0',
    T3              => '0',
    T4              => '0',
    TBYTEIN         => '0',         -- 1-bit input: Byte group tristate
    TCE             => '0'          -- 1-bit input: 3-state clock enable
);

OSERDESE2_slave: OSERDESE2
generic map (
    DATA_RATE_OQ    => "DDR",       -- DDR, SDR
    DATA_RATE_TQ    => "SDR",       -- DDR, BUF, SDR
    DATA_WIDTH      => Port_width,  -- Parallel data width (2-8,10,14)
    INIT_OQ         => '0',         -- Initial value of OQ output (1'b0,1'b1)
    INIT_TQ         => '0',         -- Initial value of TQ output (1'b0,1'b1)
    SERDES_MODE     => "SLAVE",     -- MASTER, SLAVE
    SRVAL_OQ        => '0',         -- OQ output value when SR is used (1'b0,1'b1)
    SRVAL_TQ        => '0',         -- TQ output value when SR is used (1'b0,1'b1)
    TBYTE_CTL       => "FALSE",     -- Enable tristate byte operation (FALSE, TRUE)
    TBYTE_SRC       => "FALSE",     -- Tristate byte source (FALSE, TRUE)
    TRISTATE_WIDTH  => 1            -- 3-state converter width (1,4)
) port map (
    OFB             => open,        -- 1-bit output: Feedback path for data
    OQ              => open,        -- 1-bit output: Data path output
    -- SHIFTOUT1 / SHIFTOUT2: 1-bit (each) output: Data output expansion (1-bit each)
    SHIFTOUT1       => shift1,
    SHIFTOUT2       => shift2,
    TBYTEOUT        => open,        -- 1-bit output: Byte group tristate
    TFB             => open,        -- 1-bit output: 3-state control
    TQ              => open,        -- 1-bit output: 3-state control
    CLK             => clk_ser,     -- 1-bit input: High speed clock
    CLKDIV          => clk_div,     -- 1-bit input: Divided clock
    -- D1 - D8: 1-bit (each) input: Parallel data inputs (1-bit each)
    D1              => '0',
    D2              => '0',
    D3              => data_to_send(8),
    D4              => data_to_send(9),
    D5              => data_to_send(10),
    D6              => data_to_send(11),
    D7              => data_to_send(12),
    D8              => data_to_send(13),
    OCE             => '1',         -- 1-bit input: Output data clock enable
    RST             => reset_in,       -- 1-bit input: Reset
    -- SHIFTIN1 / SHIFTIN2: 1-bit (each) input: Data input expansion (1-bit each)
    SHIFTIN1        => '0',
    SHIFTIN2        => '0',
    -- T1 - T4: 1-bit (each) input: Parallel 3-state inputs
    T1              => '0',
    T2              => '0',
    T3              => '0',
    T4              => '0',
    TBYTEIN         => '0',         -- 1-bit input: Byte group tristate
    TCE             => '0'          -- 1-bit input: 3-state clock enable
);


end Behavioral;
