----------------------------------------------------------------------------------
-- Company: AGH
-- Engineer: ukasz Jele
-- 
-- Create Date: 28.01.2017 12:35:17
--
-- Module Name: TMDS Encoder
-- Project Name: RobotEYE
-- Target Devices: Nexys Video
-- Description: 
--      Encoding 8bit colour data_in to 10bit TMDS signal
--
-- Dependencies: 
--      None 
--
-- Additional Comments:
--      None
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity tmds_encoder is
    Port ( 
        data_in 	    : in   STD_LOGIC_VECTOR(7 downto 0) := x"A5";
        send_cmd        : in   STD_LOGIC := '0';
        cmd             : in   STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
        endoded_data    : out  STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
        clk             : in   STD_LOGIC := '0'
    );
end tmds_encoder;

architecture Behavioral of tmds_encoder is

    signal xored                : STD_LOGIC_VECTOR (8 downto 0);
    signal xnored               : STD_LOGIC_VECTOR (8 downto 0);
    
    signal xored_buff           : STD_LOGIC_VECTOR (8 downto 0);
    signal xnored_buff          : STD_LOGIC_VECTOR (8 downto 0);
    signal data_in_buff         : STD_LOGIC_VECTOR (7 downto 0);
    signal cmd_buff             : STD_LOGIC_VECTOR (9 downto 0);
    signal send_cmd_buff        : STD_LOGIC;
    
    signal cmd_2_buff           : STD_LOGIC_VECTOR (9 downto 0);
    signal send_cmd_2_buff      : STD_LOGIC;
    
    signal ones                      : STD_LOGIC_VECTOR (3 downto 0);
    
    
    signal data_word                 : STD_LOGIC_VECTOR (8 downto 0);
    signal data_word_inv             : STD_LOGIC_VECTOR (8 downto 0);
    signal data_word_disparity       : STD_LOGIC_VECTOR (3 downto 0);
    signal data_word_buff            : STD_LOGIC_VECTOR (8 downto 0);
    signal data_word_inv_buff        : STD_LOGIC_VECTOR (8 downto 0);
    signal data_word_disparity_buff  : STD_LOGIC_VECTOR (3 downto 0);
    
    
    
    signal dc_bias              : STD_LOGIC_VECTOR (3 downto 0) := (others => '0');



begin
    -- Work our the two different encodings for the byte
    xored(0) <= data_in(0);
    xored(1) <= data_in(1) xor xored(0);
    xored(2) <= data_in(2) xor xored(1);
    xored(3) <= data_in(3) xor xored(2);
    xored(4) <= data_in(4) xor xored(3);
    xored(5) <= data_in(5) xor xored(4);
    xored(6) <= data_in(6) xor xored(5);
    xored(7) <= data_in(7) xor xored(6);
    xored(8) <= '1';
    
    xnored(0) <= data_in(0);
    xnored(1) <= data_in(1) xnor xnored(0);
    xnored(2) <= data_in(2) xnor xnored(1);
    xnored(3) <= data_in(3) xnor xnored(2);
    xnored(4) <= data_in(4) xnor xnored(3);
    xnored(5) <= data_in(5) xnor xnored(4);
    xnored(6) <= data_in(6) xnor xnored(5);
    xnored(7) <= data_in(7) xnor xnored(6);
    xnored(8) <= '0';
    
    
    process(clk)
    begin
        if rising_edge(clk) then
            xored_buff      <= xored;
            xnored_buff     <= xnored;
            data_in_buff    <= data_in;
            cmd_buff        <= cmd;
            send_cmd_buff   <= send_cmd;
        end if;
    end process;
    
    -- Count how many ones are set in data_in
    ones <= "0000" + data_in_buff(0) + data_in_buff(1) + data_in_buff(2) + data_in_buff(3)
                    + data_in_buff(4) + data_in_buff(5) + data_in_buff(6) + data_in_buff(7);
    
    -- Decide which encoding to use
    process(ones, data_in_buff(0), xnored_buff, xored_buff)
    begin
    if ones > 4 or (ones = 4 and data_in_buff(0) = '0') then
      data_word_buff     <= xnored_buff;
      data_word_inv_buff <= NOT(xnored_buff);
    else
      data_word_buff     <= xored_buff;
      data_word_inv_buff <= NOT(xored_buff);
    end if;
    end process;                                          
    
    -- Work out the DC bias of the dataword;
    data_word_disparity_buff  <= "1100" + data_word_buff(0) + data_word_buff(1) + data_word_buff(2) + data_word_buff(3) 
                                 + data_word_buff(4) + data_word_buff(5) + data_word_buff(6) + data_word_buff(7);
    
    
    process(clk)
    begin
        if rising_edge(clk) then
            data_word               <= data_word_buff;                     
            data_word_inv           <= data_word_inv_buff;      
            data_word_disparity     <= data_word_disparity_buff;
            cmd_2_buff              <= cmd_buff;
            send_cmd_2_buff         <= send_cmd_buff;
        end if;
    end process;
    
    
    
    -- Now work out what the output should be
    process(clk)
    begin
       if rising_edge(clk) then
          if send_cmd_2_buff = '1' then 
             -- In the control periods, all values have and have balanced bit count
            
                endoded_data <= cmd_2_buff;
    
             dc_bias <= (others => '0');
          else
             if dc_bias = "00000" or data_word_disparity = 0 then
                -- dataword has no disparity
                if data_word(8) = '1' then
                   endoded_data <= "01" & data_word(7 downto 0);
                   dc_bias <= dc_bias + data_word_disparity;
                else
                   endoded_data <= "10" & data_word_inv(7 downto 0);
                   dc_bias <= dc_bias - data_word_disparity;
                end if;
             elsif (dc_bias(3) = '0' and data_word_disparity(3) = '0') or 
                   (dc_bias(3) = '1' and data_word_disparity(3) = '1') then
                endoded_data <= '1' & data_word(8) & data_word_inv(7 downto 0);
                dc_bias <= dc_bias + data_word(8) - data_word_disparity;
             else
                endoded_data <= '0' & data_word;
                dc_bias <= dc_bias - data_word_inv(8) + data_word_disparity;
             end if;
          end if;
       end if;
    end process;      
    
end Behavioral;
