----------------------------------------------------------------------------------
-- Company: AGH
-- Engineer: ukasz Jele
-- 
-- Create Date: 31.01.2017 01:39:41
--
-- Module Name: VGA_to_DVI
-- Project Name: RobotEYE
-- Target Devices: Nexys Video
-- Description: 
--      Module converts VGA signal to DVI.
-- 
-- Dependencies: 
--      MMCM - simple clock managment
-- 
-- Additional Comments:
-- 
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity VGA_to_DVI is
    Port ( 
        clk_in          : in    STD_LOGIC;
        reset_in        : in    STD_LOGIC;
        
        VGA_red_in      : in    STD_LOGIC_VECTOR(7 downto 0);
        VGA_green_in    : in    STD_LOGIC_VECTOR(7 downto 0);
        VGA_blue_in     : in    STD_LOGIC_VECTOR(7 downto 0);
        VGA_hsync_in    : in    STD_LOGIC;
        VGA_vsync_in    : in    STD_LOGIC;
        VGA_blank_in    : in    STD_LOGIC;
        
        DVI_clk_out     : out   STD_LOGIC;
        DVI_ch_out      : out   STD_LOGIC_VECTOR(2 downto 0)
    );
end VGA_to_DVI;

architecture Behavioral of VGA_to_DVI is


--##--##--##--##--## Clocking ##--##--##--##--##--
    component dv_seri_clk
        port(
            clk_div_out       : out    std_logic;
            clk_diff_out      : out    std_logic;
            -- Status and control signals
            reset             : in     std_logic;
            locked            : out    std_logic;
            clk_in1           : in     std_logic
        );
    end component;
    
    
--##--##--##--##--## TMDS Encoder ##--##--##--##--##--
    component tmds_encoder is
        Port ( 
            data_in         : in   STD_LOGIC_VECTOR(7 downto 0) := x"A5";
            send_cmd        : in   STD_LOGIC := '0';
            cmd             : in   STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
            endoded_data    : out  STD_LOGIC_VECTOR(9 downto 0) := (others => '0');
            clk             : in   STD_LOGIC := '0'
        );
    end component tmds_encoder;
    
    
    
--##--##--##--##--## Serializer ##--##--##--##--##--
    component serializer is
        Generic (
            Port_width      : integer := 10         -- (2-8,10,14)
        );
        Port ( 
            clk_ser         : in    STD_LOGIC;
            clk_div         : in    STD_LOGIC;
            parallel_in     : in    STD_LOGIC_VECTOR( Port_width - 1 downto 0 );
            serial_out      : out   STD_LOGIC;
            reset_in        : in    STD_LOGIC
        );
    end component serializer;

--##--##--##--##--## FIFO to change clk ##--##--##--##--##--

    COMPONENT fifo_change_clk_domain
        PORT (
            rst             : IN STD_LOGIC;
            wr_clk          : IN STD_LOGIC;
            rd_clk          : IN STD_LOGIC;
            din             : IN STD_LOGIC_VECTOR(26 DOWNTO 0);
            wr_en           : IN STD_LOGIC;
            rd_en           : IN STD_LOGIC;
            dout            : OUT STD_LOGIC_VECTOR(26 DOWNTO 0);
            full            : OUT STD_LOGIC;
            empty           : OUT STD_LOGIC
        );
    END COMPONENT;
    
    signal fifo_data_in     : STD_LOGIC_VECTOR(26 DOWNTO 0);
    signal fifo_data_out    : STD_LOGIC_VECTOR(26 DOWNTO 0);
    signal fifo_data_buf    : STD_LOGIC_VECTOR(26 downto 0);
    
    signal w_fifo_empty     : STD_LOGIC;
    signal w_fifo_empty_n   : STD_LOGIC;
    
    
--##--##--##--##--## Functions ##--##--##--##--##--


--##--##--##--##--##--##--##--
--**-- Function generating constant pattern for DVI 
--##--##--##--##--##--##--##--
--**-- IN:
--**--      D(1) => VSYNC                         
--**--      D(0) => HSYNC
--**-- OUT:
--**--      Pattern for CTL period

    function min_DVI (D : STD_LOGIC_VECTOR(1 downto 0)) return STD_LOGIC_VECTOR is
    begin
        case D is
            when "00"   => return b"1101010100";
            when "01"   => return b"0010101011";
            when "10"   => return b"0101010100";
            when others => return b"1010101011";
        end case;
    end min_DVI;
    


--##--##--##--##--## GLOBAL Signal ##--##--##--##--##--

--**-- Clocking signals --**-- 
    signal w_clk_div        : STD_LOGIC;
    signal w_clk_diff       : STD_LOGIC;
    
--**-- Data signals --**-- 
    signal w_clk_locked     : STD_LOGIC;
    signal w_clk_locked_n   : STD_LOGIC;
    
    signal tmds_ch0         : STD_LOGIC_VECTOR(9 downto 0);
    signal tmds_ch1         : STD_LOGIC_VECTOR(9 downto 0);
    signal tmds_ch2         : STD_LOGIC_VECTOR(9 downto 0);
    
    --** VSYNC & HSYNC 
    signal fl_VGA_VHsync     : STD_LOGIC_VECTOR(1 downto 0);
    signal fl_VGA_red        : STD_LOGIC_VECTOR(7 downto 0);
    signal fl_VGA_green      : STD_LOGIC_VECTOR(7 downto 0);
    signal fl_VGA_blue       : STD_LOGIC_VECTOR(7 downto 0);
    signal fl_VGA_blank      : STD_LOGIC;
    
begin

--##--##--##--##--## GLOBAL conn ##--##--##--##--##--
    DVI_clk_out         <= w_clk_div;
    
    w_clk_locked_n      <= not w_clk_locked;
    
    --** FIFO
    fifo_data_in        <= VGA_vsync_in & VGA_hsync_in & VGA_blank_in & VGA_blue_in & VGA_green_in & VGA_red_in;
    fl_VGA_VHsync       <= fifo_data_out(26 downto 25);
    fl_VGA_blank        <= fifo_data_out(24);
    fl_VGA_blue         <= fifo_data_out(23 downto 16);
    fl_VGA_green        <= fifo_data_out(15 downto 8);
    fl_VGA_red          <= fifo_data_out(7 downto 0);
    
    process(w_clk_div)
    begin
        if rising_edge(w_clk_div) then
            w_fifo_empty_n      <= not w_fifo_empty;
            fifo_data_out       <= fifo_data_buf;
        end if;
    end process;

--##--##--##--##--## Registering inputs ##--##--##--##--##--


--first_reg_inputs:
--    process(clk_in)
--    begin
--        if rising_edge(clk_in) then
--            fl_VGA_red      <= VGA_red_in;                 
--            fl_VGA_green    <= VGA_green_in;               
--            fl_VGA_blue     <= VGA_blue_in;                
--            fl_VGA_blank    <= VGA_blank_in;               
--            fl_VGA_VHsync   <= VGA_vsync_in & VGA_hsync_in;
--        end if;
--    end process;

change_clk_domain: 
    fifo_change_clk_domain PORT MAP (
        rst => reset_in,
        wr_clk => clk_in,
        rd_clk => w_clk_div,
        din => fifo_data_in,
        wr_en => w_clk_locked,
        rd_en => w_fifo_empty_n,
        dout => fifo_data_buf,
        full => open,
        empty => w_fifo_empty
    );
    
    
--sec_reg_inputs:
--    process(w_clk_div)
--    begin
--        if rising_edge(w_clk_div) then
--            l_VGA_red       <= fl_VGA_red;  
--            l_VGA_green     <= fl_VGA_green;  
--            l_VGA_blue      <= fl_VGA_blue;  
--            l_VGA_blank     <= fl_VGA_blank;  
--            l_VGA_VHsync    <= fl_VGA_VHsync;  
--        end if;
--    end process;


--##--##--##--##--## Clocking  ##--##--##--##--##--
    
hdmi_ser_clk: 
    dv_seri_clk port map ( 
        clk_div_out     => w_clk_div,
        clk_diff_out    => w_clk_diff,            
        reset           => reset_in,
        locked          => w_clk_locked,
        clk_in1         => clk_in
    );
    
--##--##--##--##--## TMDS encoding ##--##--##--##--##--
    
tmds_ch0_b:
    tmds_encoder Port map( 
        data_in         => fl_VGA_blue,
        send_cmd        => fl_VGA_blank,
        cmd             => min_DVI(fl_VGA_VHsync),
        endoded_data    => tmds_ch0,
        clk             => w_clk_div
    );
    
tmds_ch1_g:
    tmds_encoder Port map( 
        data_in         => fl_VGA_green,
        send_cmd        => fl_VGA_blank,
        cmd             => min_DVI("00"),
        endoded_data    => tmds_ch1,
        clk             => w_clk_div
    );
    
tmds_ch2_r:
    tmds_encoder Port map( 
        data_in         => fl_VGA_red,
        send_cmd        => fl_VGA_blank,
        cmd             => min_DVI("00"),
        endoded_data    => tmds_ch2,
        clk             => w_clk_div
    );

--##--##--##--##--## Serialize data ##--##--##--##--##--

ser_ch0_b: 
    serializer generic map(
        Port_width      => 10
    ) port map (
        clk_ser         => w_clk_diff,
        clk_div         => w_clk_div,
        parallel_in     => tmds_ch0,
        serial_out      => DVI_ch_out(0),
        reset_in        => w_clk_locked_n
    );
    
ser_ch1_g: 
    serializer generic map(
        Port_width      => 10
    ) port map (
        clk_ser         => w_clk_diff,
        clk_div         => w_clk_div,
        parallel_in     => tmds_ch1,
        serial_out      => DVI_ch_out(1),
        reset_in        => w_clk_locked_n
    );
    
ser_ch2_r: 
    serializer generic map(
        Port_width      => 10
    ) port map (
        clk_ser         => w_clk_diff,
        clk_div         => w_clk_div,
        parallel_in     => tmds_ch2,
        serial_out      => DVI_ch_out(2),
        reset_in        => w_clk_locked_n
    );
    
--ser_clk: 
--    serializer generic map(
--        Port_width      => 10
--    ) port map (
--        clk_ser         => w_clk_diff,
--        clk_div         => w_clk_div,
--        parallel_in     => "0000011111",
--        serial_out      => DVI_clk_out,
--        reset_in        => w_clk_locked_n
--    );

end Behavioral;
